#include "StdAfx.h"
#include ".\nbinaryresourceloader.h"

#define TIXML_USE_STL
#include "tinyxml.h"

#include "XMLUtil.h"
#include "NPFileData.h"
#include "NPMemoryData.h"
#include "GDIPlusImage.h"
#include <atlpath.h>

static const TCHAR	LANG[] = _T("lang");
static const TCHAR	RESOURCE[] = _T("resource");
static const TCHAR	IMAGE[] = _T("image");
static const TCHAR	ID[] = _T("id");
static const TCHAR	VALUE[] = _T("value");
static const TCHAR  _CONST[] = _T("const");

CNBinaryResourceLoader::CNBinaryResourceLoader(void)
{
}

CNBinaryResourceLoader::~CNBinaryResourceLoader(void)
{
}
BOOL CNBinaryResourceLoader::Init(CNPUnzipFile* zipimage)
{
	__super::Init(zipimage);
	
	m_ZipImage->GoToFirstFile();

	UnzipFileInfo fileInfo;
	char szCurrentFileName[_MAX_PATH+1];

	BOOL bNextFile = TRUE;
	while (bNextFile)
	{
		m_ZipImage->GetCurrentFileInfo(&fileInfo, szCurrentFileName, _MAX_PATH, NULL, 0, NULL, 0);
		if (strncmp("binary", szCurrentFileName, strlen("binary")) == 0 &&
			strcmp(".xml", ATLPath::FindExtension(szCurrentFileName)) == 0)
		{
			LONG lFileSize = fileInfo.m_lUncompressedSize;
			if (lFileSize > 0)
			{
				CNByteArray bufArray;
				ReadCurrentZipFile(bufArray, lFileSize);

				if (bufArray.GetSize() > 0)
				{
					_ParseXML(bufArray.GetBuffer());
				}
			}
		}
		bNextFile = m_ZipImage->GoToNextFile();
	}
	m_ZipImage->GoToFirstFile();

	return TRUE;
}
BOOL CNBinaryResourceLoader::Close()
{
	/*POSITION pos = m_mapImage.GetStartPosition();
	while (pos)
	{
		IImage* pImage = m_mapImage.GetNextValue(pos);
		if (pImage)
		{
			delete pImage;
		}
	}*/
	return TRUE;
}
///< get binary
BOOL CNBinaryResourceLoader::GetBinary(CString id, CNByteArray& ByteArray)
{
	CAtlMap< CString, CString >::CPair* pPair = m_mapPath.Lookup(id);
	if (pPair != NULL)
	{
		CString strPath = pPair->m_value;

		USES_CONVERSION;
		if (m_ZipImage->FindFileInZip(CT2A(strPath)))
		{
			UnzipFileInfo fileInfo;

			//   о ´.
			int nResult = m_ZipImage->GetCurrentFileInfo(&fileInfo);

			LONG lFileSize = fileInfo.m_lUncompressedSize;
			BYTE* bufImage = ByteArray.Alloc(lFileSize);
			m_ZipImage->OpenCurrentFile();

			DWORD dwIndex = 0;
			BYTE buffer[256];
			int size_read;
			do
			{		
				size_read = m_ZipImage->ReadCurrentFile(buffer, 256);
				CopyMemory(bufImage+dwIndex, buffer, size_read);
				dwIndex += size_read;
			}
			while (size_read>0);	

			m_ZipImage->CloseCurrentFile();

			return TRUE;
		}
	}
	return FALSE;
}
void CNBinaryResourceLoader::FreeImage(IImage* pImage)
{
	if (pImage != NULL)
	{
		delete pImage;
	}
}
IImage*	CNBinaryResourceLoader::GetImage(LPCTSTR pszcConst)
{
	IImage* pImage = NULL;
	CString id(pszcConst);
	CNByteArray data;
	if (this->GetBinary(id, data))
	{
		CGDIPlusImage* pBitmapImage = new CGDIPlusImage;
		//CBitmapImage* pBitmapImage = new CBitmapImage;
		if (pBitmapImage->Load(data.GetBuffer(), data.GetSize()))
		{
			pImage = reinterpret_cast<IImage*>(pBitmapImage);			
		}
		else
		{
			delete pBitmapImage;
			pImage = NULL;
		}
	}
	/*
	CAtlMap<CString, IImage*>::CPair* pPair = m_mapImage.Lookup(id);
	if (pPair)
	{
		pImage = pPair->m_value;
	}
	if (pImage == NULL)
	{
		CString strPath = m_mapPath[id];
		if (strPath.GetLength() > 0)
		{
			USES_CONVERSION;
			if (m_ZipImage->FindFileInZip(CT2A(strPath)))
			{
				UnzipFileInfo fileInfo;

				//   о ´.
				int nResult = m_ZipImage->GetCurrentFileInfo(&fileInfo);
				
				LONG lFileSize = fileInfo.m_lUncompressedSize;
				BYTE* bufImage = new BYTE[lFileSize];
				m_ZipImage->OpenCurrentFile();

				DWORD dwIndex = 0;
				BYTE buffer[256];
				int size_read;
				do
				{		
					size_read = m_ZipImage->ReadCurrentFile(buffer, 256);
					CopyMemory(bufImage+dwIndex, buffer, size_read);
					dwIndex += size_read;
				}
				while (size_read>0);	
				
				m_ZipImage->CloseCurrentFile();

				CGDIPlusImage* pBitmapImage = new CGDIPlusImage;
				//CBitmapImage* pBitmapImage = new CBitmapImage;
				if (pBitmapImage->Load(bufImage, lFileSize))
				{
					pImage = reinterpret_cast<IImage*>(pBitmapImage);
					m_mapImage.SetAt(id, pImage);
				}
				else
				{
					delete pBitmapImage;
				}
				delete[] bufImage;
			}
		}
	}
	*/
	return pImage;
}
BOOL CNBinaryResourceLoader::_ParseXML(const BYTE * pBuf)
{
	USES_CONVERSION;

	BOOL bSuccess = FALSE;
	TiXmlDocument document;
	document.Parse(reinterpret_cast<LPCSTR>(pBuf));
	TiXmlElement* pRoot = document.FirstChildElement(T2A(RESOURCE));
	if (!pRoot) return FALSE;
	TiXmlElement* pLangNode = pRoot->FirstChildElement(T2A(LANG));

	static const int MAX_ID = 32;
	static const int MAX_VALUE = 1024;
	static int nID = 0;

	TCHAR	szID[MAX_ID+1] = {0};
	TCHAR	szValue[MAX_VALUE+1] = {0};
	TCHAR	szConst[MAX_VALUE+1] = {0};
	TiXmlElement* pImageNode = pRoot->FirstChildElement(T2A(IMAGE));
	while (pImageNode)
	{
		//XMLUtil::GetAttrValue(pImageNode, T2A(ID), szID, MAX_ID);
		_stprintf_s(szID, MAX_ID, _T("%d"), ++nID);
		XMLUtil::GetAttrValue(pImageNode, T2A(_CONST), szConst, MAX_VALUE);
		XMLUtil::GetAttrValue(pImageNode, T2A(VALUE), szValue, MAX_VALUE);

		m_mapPath.SetAt(CString(szConst), CString(szValue));
		
		pImageNode = pImageNode->NextSiblingElement(T2A(IMAGE));
	}
	return bSuccess;
}
BOOL CNBinaryResourceLoader::GetBinaryList(CAtlMap< CString, CString >& lstBinary)
{
	POSITION pos = m_mapPath.GetStartPosition();
	while (pos)
	{
		CAtlMap< CString, CString >::CPair* pPair = m_mapPath.GetNext(pos);
		lstBinary.SetAt(pPair->m_key, pPair->m_value);
	}
	return TRUE;
}